import com.haxepunk.Scene;
import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.graphics.*;
import com.haxepunk.masks.*;
import com.haxepunk.utils.*;

class MainScene extends Scene
{

	public var player: Player;

	public var healthText: Text;
	public var killsText: Text;
	public var kills:Int = 0;


	public override function begin()
	{

		// LOad monsters
		Monster.loadMonsters("monsters.xml");


		var map:Entity = generateMap(0xff751, 100, "tiles0.png");
		add( map );


		player = new Player(200, 100);

		add( player );


		// Spawn some enemies
		for(i in 0...10) {
			spawn( "worm", HXP.rand(3600), HXP.rand(3600));
		}


		HXP.console.watch( [ "uid", "health", "state", "target", "stateTime", "cooldown"]);


		healthText = new Text("Health: 100");
		killsText = new Text("Kills: 0");
		healthText.scrollX = healthText.scrollY = 0;
		killsText.scrollX = killsText.scrollY = 0;
		addGraphic(healthText, -1, 0,0);
		addGraphic(killsText, -1, HXP.width-100,0);

	}

	public function spawn(type:String, x:Int, y:Int) {
		var m = Monster.create(type, x, y);
		if(m!=null && fixSpawn(m)) add( m );
	}


	public function fixSpawn(e:Entity) {
		var r:Int = 1;
		while(e.collide("solid", e.x, e.y)!=null) {
			switch(r&3) {
			case 0: e.x += r*16;
			case 1: e.y += r*16;
			case 2: e.x -= r*16;
			case 3: e.y -= r*16;
			}
			HXP.clampInRect(e, 0,0,3200,3200);
			++r;
			if(r>40) return false; // give up
		}
		return true;
	}

	


	public override function update() {
		super.update();
		camera.x = player.x - HXP.halfWidth;
		camera.y = player.y - HXP.halfHeight;
		HXP.clampInRect(camera, 0, 0, 3600-HXP.width, 3600-HXP.height);

		// restart
		if(Input.pressed(Key.R)) {
			HXP.world = new MainScene();
		}


		if(player.health<=0) return; // Dead

		// Spawn ...
		if(Input.mousePressed && collideRect("solid", mouseX-6, mouseY-6, 12,12)==null) {
			spawn("snake", mouseX, mouseY);
		}

		//switch worlds
		if(Input.pressed(Key.SPACE)) {
			removeAll();
			var n: Int = HXP.rand(3);
			var m = generateMap( HXP.rand(0xffffff), 100 , "tiles" + n + ".png");
			add(m);
			add(player);

			var t = ["worm", "lizard", "spider"];
			for(i in 0...16) spawn(t[n], HXP.rand(3200), HXP.rand(3200));

			addGraphic(healthText, -1, 0,0);
			addGraphic(killsText, -1, HXP.width-100,0);
		}

	}


	public function getTileCode(map:Tilemap, x:Int, y:Int) : UInt {
		var c:UInt = 0;
		var s:Int = map.rows-1;
		if(x==0 || y==0 || map.getTile(x-1, y-1) == 9) c |= 0x020;
		if(y==0 ||         map.getTile(x,   y-1) == 9) c |= 0x008;
		if(x==s || y==0 || map.getTile(x+1, y-1) == 9) c |= 0x040;
		if(x==0 ||         map.getTile(x-1, y)   == 9) c |= 0x002;
		if(                map.getTile(x,   y)   == 9) c |= 0x001;
		if(x==s ||         map.getTile(x+1, y)   == 9) c |= 0x004;
		if(x==0 || y==s || map.getTile(x-1, y+1) == 9) c |= 0x080;
		if(y==s ||         map.getTile(x,   y+1) == 9) c |= 0x010;
		if(x==s || y==s || map.getTile(x+1, y+1) == 9) c |= 0x100;
		return c;
	}


	public function generateMap(seed:Int, size:UInt, tileset:String) : Entity {
		//trace("generating map");
		
		// Infinite levels?
		// Perhaps join them together later ...


		var map:Tilemap = new Tilemap("graphics/"+tileset, 32*size, 32*size, 32,32 );

		map.setRect(0,0,size,size, 9);
		map.setRect(1,1,size-1,size-1, 0);

		// Draw some boxes
		HXP.randomSeed = seed;
		var count:Int = HXP.rand(100)+50;
		var max:Int = HXP.rand(20);
		for(i in 1...count) {
			
			var x:Int = HXP.rand(size-1);
			var y:Int = HXP.rand(size-1);
			var w:Int = HXP.rand(max)+2;
			var h:Int = HXP.rand(max)+2;
			map.setRect(x,y,w,h, 9);
		}

		var m = new Map();
		m.set( 0x15, 1);
		m.set( 0x13, 2);
		m.set( 0x15, 3);
		m.set( 0x17, 4);
		m.set( 0x1b, 6);
		m.set( 0x0d, 7);
		m.set( 0x13, 11);
		m.set( 0x1d, 12);
		m.set( 0x0f, 14);
		m.set( 0x0b, 15);

		m.set( 0x1df, 5);
		m.set( 0x17f, 8);
		m.set( 0x1ff, 9);
		m.set( 0x1bf, 10);
		m.set( 0x0ff, 13);


		// Beautify
		var code = new Array();
		for(y in 0...size) for(x in 0...size) code.push( getTileCode(map, x, y) );
		for(y in 0...size) for(x in 0...size) {
			var c:UInt = code[x+y*size];
			
			// Translate
			if(m.exists(c)) map.setTile(x, y, m.get(c));
			else if(m.exists(c&0x1f)) map.setTile(x,y,m.get(c&0x1f));
		}


		// Create mask
		var mask:Grid = new Grid(32*size, 32*size, 32, 32);
		for(y in 0...size) for(x in 0...size) {
			mask.setTile( x, y, code[x+y*size] & 1 == 1);
		}
		

		var e:Entity = new Entity(0,0,map,mask);
		e.type = "solid";
		return e;

	}

}
